<?php

namespace Drupal\Tests\block_form_alter\Functional;

use Drupal\block_content\Entity\BlockContentType;
use Drupal\Tests\BrowserTestBase;

/**
 * Tests functionality of block_form_alter module.
 *
 * @group block_form_alter
 */
class BlockFormAlterTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'block_content',
    'block_form_alter',
    'block_form_alter_test',
    'layout_builder',
    'node',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();

    // Place certain blocks.
    $this->drupalPlaceBlock('system_menu_block:account');
    $this->drupalPlaceBlock('local_tasks_block');
    $this->drupalPlaceBlock('system_branding_block', [
      'label' => 'Branding Block',
      'id' => 'systembrandingblock',
    ]);

    // Create a custom block type.
    $bundle = BlockContentType::create([
      'id' => 'test_block',
      'label' => 'Test Block',
      'revision' => FALSE,
    ]);
    $bundle->save();
    block_content_add_body_field('test_block');

    // Create a content type and node.
    $this->createContentType([
      'type' => 'bundle_with_section_field',
      'name' => 'Bundle with section field',
    ]);
    $this->createNode([
      'type' => 'bundle_with_section_field',
      'title' => 'The first node title',
      'body' => [
        [
          'value' => 'The first node body',
        ],
      ],
    ]);

    // Create admin user.
    $this->adminUser = $this->drupalCreateUser([
      'access administration pages',
      'administer blocks',
      'bypass node access',
      'administer node display',
    ]);
  }

  /**
   * Tests the Block Form Alter functions.
   */
  public function testBlockFormAlter() {
    $assert_session = $this->assertSession();
    $page = $this->getSession()->getPage();

    $this->drupalLogin($this->adminUser);

    // Verify hook_block_plugin_form_alter() is modifying block forms
    // rendered by block_form.
    $this->drupalGet('/admin/structure/block/manage/systembrandingblock');
    $assert_session->checkboxNotChecked('settings[block_branding][use_site_logo]');

    // Verify hook_block_type_form_alter() is modifying custom block forms
    // rendered by block_content.
    $this->drupalGet('/block/add/test_block');
    $assert_session->fieldValueEquals('body[0][value]', 'test body string');

    // Enable layout override of test content type.
    // "Borrowed" from LayoutBuilderTest.php.
    $this->drupalGet('admin/structure/types/manage/bundle_with_section_field/display/default');
    $this->drupalPostForm(NULL, ['layout[enabled]' => TRUE], 'Save');
    $this->drupalPostForm(NULL, ['layout[allow_custom]' => TRUE], 'Save');
    // @todo This should not be necessary.
    $this->container->get('entity_field.manager')->clearCachedFieldDefinitions();

    // Update admin user permissions with Layout Builder permissions.
    $this->adminUser = $this->drupalCreateUser([
      'access administration pages',
      'administer blocks',
      'bypass node access',
      'administer node display',
      'configure editable bundle_with_section_field node layout overrides',
      'create and edit custom blocks',
    ]);
    $this->drupalLogin($this->adminUser);

    // Override layout of test node.
    $this->drupalGet('node/1');
    $page->clickLink('Layout');
    $page->pressButton('Save layout');

    // Verify hook_block_plugin_form_alter() is modifying block forms
    // rendered by layout_builder.
    $this->drupalGet('/layout_builder/add/block/overrides/node.1/0/content/system_branding_block');
    $assert_session->checkboxNotChecked('settings[block_branding][use_site_logo]');

    // Verify hook_block_type_form_alter() is modifying custom block forms
    // rendered by layout_builder.
    $this->drupalGet('/layout_builder/add/block/overrides/node.1/0/content/inline_block:test_block');
    $assert_session->fieldValueEquals('settings[block_form][body][0][value]', 'test body string');
  }

}
